#****************************************************************************
#**
#**  File     :  /cdimage/units/URA0107/URA0107_script.lua
#**  Author(s):  John Comes, David Tomandl
#**
#**  Summary  :  Cybran T1 Air Transport Script
#**
#**  Copyright  2006 Gas Powered Games, Inc.  All rights reserved.
#****************************************************************************

local CAirUnit = import('/lua/cybranunits.lua').CAirUnit
local explosion = import('/lua/defaultexplosions.lua')
local util = import('/lua/utilities.lua')

URA0107 = Class(CAirUnit) {
    Weapons = {
    },

    AirDestructionEffectBones = { 'Left_Exhaust', 'Right_Exhaust', 'Char04', 'Char03', 'Char02', 'Char01',
                                  'Front_Left_Leg03_B02', 'Front_Right_Leg03_B02', 'Front_Left_Leg01_B02', 'Front_Right_Leg01_B02',
                                  'Right_AttachPoint01', 'Right_AttachPoint02', 'Right_AttachPoint03', 'Right_AttachPoint04',
                                  'Left_AttachPoint01', 'Left_AttachPoint02', 'Left_AttachPoint03', 'Left_AttachPoint04', },

    BeamExhaustIdle = '/effects/emitters/missile_exhaust_fire_beam_05_emit.bp',
    BeamExhaustCruise = '/effects/emitters/missile_exhaust_fire_beam_04_emit.bp',

    OnStopBeingBuilt = function(self,builder,layer)
        
        self.AnimManip = CreateAnimator(self)
        self.Trash:Add(self.AnimManip)
        self.AnimManip:PlayAnim(self:GetBlueprint().Display.AnimationTakeOff, false):SetRate(1)
        
        LOG('*ATTACHING UNITS TO TRANS!!')
        local position = self:GetPosition()
		
		self.Station01 = CreateUnitHPR('url0106', self:GetArmy(), position.x, position.y, position.z, 0, 0, 0)
		self.Station01:AttachBoneTo(0, self, 'Attachpoint_sml_01')
		
		self.Station02 = CreateUnitHPR('url0106', self:GetArmy(), position.x, position.y, position.z, 0, 0, 0)
		self.Station02:AttachBoneTo(0, self, 'Attachpoint_sml_02')
		
		self.Station03 = CreateUnitHPR('url0106', self:GetArmy(), position.x, position.y, position.z, 0, 0, 0)
		self.Station03:AttachBoneTo(0, self, 'Attachpoint_sml_03')
		
		self.Station04 = CreateUnitHPR('url0106', self:GetArmy(), position.x, position.y, position.z, 0, 0, 0)
		self.Station04:AttachBoneTo(0, self, 'Attachpoint_sml_04')
		
		self.Station05 = CreateUnitHPR('url0106', self:GetArmy(), position.x, position.y, position.z, 0, 0, 0)
		self.Station05:AttachBoneTo(0, self, 'Attachpoint_sml_05')
		
		self.Station06 = CreateUnitHPR('url0106', self:GetArmy(), position.x, position.y, position.z, 0, 0, 0)
		self.Station06:AttachBoneTo(0, self, 'Attachpoint_sml_06')
		CAirUnit.OnStopBeingBuilt(self,builder,layer)
		
    end,

    # When one of our attached units gets killed, detach it
    OnAttachedKilled = function(self, attached)
        attached:DetachFrom()
    end,

    OnKilled = function(self, instigator, type, overkillRatio)
        CAirUnit.OnKilled(self, instigator, type, overkillRatio)
        # TransportDetachAllUnits takes 1 bool parameter. If true, randomly destroys some of the transported
        # units, otherwise successfully detaches all.
        self:TransportDetachAllUnits(true)
    end,

    OnMotionVertEventChange = function(self, new, old)
        #LOG( 'OnMotionVertEventChange, new = ', new, ', old = ', old )
        CAirUnit.OnMotionVertEventChange(self, new, old)
        #Aborting a landing
        if ((new == 'Top' or new == 'Up') and old == 'Down') then
            self.AnimManip:SetRate(-1)
        elseif (new == 'Down') then
            self.AnimManip:PlayAnim(self:GetBlueprint().Display.AnimationLand, false):SetRate(1.5)
#NOTE: We need to add funtionality to fit this guy on the platform            
#        elseif (new == 'Bottom') then
#            self.AnimManip:PlayAnim(self:GetBlueprint().Display.AnimationIdle, true):SetRate(1)
        elseif (new == 'Up') then
            self.AnimManip:PlayAnim(self:GetBlueprint().Display.AnimationTakeOff, false):SetRate(1)
        end
    end,

    # Override air destruction effects so we can do something custom here
    CreateUnitAirDestructionEffects = function( self, scale )
        self:ForkThread(self.AirDestructionEffectsThread, self )
    end,

    AirDestructionEffectsThread = function( self )
        local numExplosions = math.floor( table.getn( self.AirDestructionEffectBones ) * 0.5 )
        for i = 0, numExplosions do
            explosion.CreateDefaultHitExplosionAtBone( self, self.AirDestructionEffectBones[util.GetRandomInt( 1, numExplosions )], 0.5 )
            WaitSeconds( util.GetRandomFloat( 0.2, 0.9 ))
        end
    end,
}

TypeClass = URA0107